#!/usr/bin/env python

import sys
from proper import *

def kronecker_delta(x,y):
    if x == y:
        return 1
    else:
        return 0

# Formula 11
def t2(n, delta):
	return 2*n**2 - 5*n + 1 - sum(d**2 for d in delta)//2;

# Formula 30
def xx_formula(T, n, X_2):
    return T*(X_2 - 2*n + 2)//(n - 1)

# Formula A.7
def xyx_d0_d(T, n, X_2, X_3, d_0, d, alpha_d_0, alpha_d):
    tmp = 6*d**3 + 2*(3*d - 10)*d_0**2 + 6*d_0**3 + (d + d_0 - 10)*n**2 - (3*X_2 - 8)*d - 20*d**2 + (6*d**2 - 3*X_2 - 20*d + 8)*d_0 - (4*d**2 + (4*d - 21)*d_0 + 4*d_0**2 - 2*X_2 - 21*d + 18)*n + 10*X_2 - 2*X_3 + 16
    return (T*alpha_d_0*(alpha_d - kronecker_delta(d,d_0))*tmp)//((n - 1)*(n - 2)*(n - 3)*(n - 4))

# Formula A.8
def xyx_formula(T, n, X_2, X_3):
    return (T*(2*X_2*(n + 1) - 6*n**2 - 2*X_3 + 10*n - 4))//((n - 1)*(n - 2))
    
# Formula B.6
def xyzx_formula(T, n, X_2, X_3, X_4):
    return (T*(-10*n**3 - 12*n**2 + 50*n - 28 + 3*(n**2 + 9*n - 4)*X_2 - 3*X_2**2 - 2*X_3*(3*n + 7) + 6*X_4))//((n - 1)*(n - 2)*(n - 3))
    
def main():
    if (len(sys.argv) <= 1):
        print("missing argument")
        sys.exit(1)

    n = int(sys.argv[1])

    G2 = {}
    for p in partitions(n-2):
        delta = degreesequence(p)
        alpha = occupations(n,delta)
        
        print("# delta = %s" % sorted(delta))
        print("# alpha = %s" % alpha)
        
        # Formula 8
        T = multinomial(p)*multinomial(alpha)//n
        
        # Formula A.1
        X_2 = sum(d**2*alpha[d] for d in range(1,n))
        X_3 = sum(d**3*alpha[d] for d in range(1,n))
        X_4 = sum(d**4*alpha[d] for d in range(1,n))
        
        xx_sum = xx_formula(T, n, X_2)
        xyx_sum = xyx_formula(T, n, X_2, X_3)
        xyzx_sum = xyzx_formula(T, n, X_2, X_3, X_4)
        
        xyx = {}
        for d_0 in range(1,n):
            for d in range(d_0,n):
                tmp = xyx_d0_d(T, n, X_2, X_3, d_0, d, alpha[d_0], alpha[d])
                xyx.setdefault((d_0,d),0)
                if d_0 == d:
                    xyx[(d_0,d)] += tmp
                else:
                    xyx[(d_0,d)] += 2*tmp
            
        t = t2(n, delta)
        print("#    t = %s" % t)
        
        for k in sorted(xyx):
            if xyx[k] > 0:
                t_corrected = t - (k[0]+k[1]-2) # Formula 13
                G2.setdefault(t_corrected, 0)
                G2[t_corrected] += 2**(n-5)*xyx[k] # Table 1
                print("#    xyx[%s,%s] = %s (t=%s)" % (k[0],k[1],xyx[k],t_corrected))
        

        print("#    xx = %s (t=%s)" % (xx_sum, t+1))
        print("#    xyx_sum = %s" % xyx_sum)
        print("#    xyzx = %s (t=%s)" % (xyzx_sum, t-1))
        print("#    other = %s (t=%s)" % (((n-2)*T - xx_sum - xyx_sum - xyzx_sum),t))
        
        for i in (t-1,t,t+1):
            G2.setdefault(i,0)
        
        # Table 1    
        error_free = (n-2)*T - xx_sum - xyx_sum//2 - xyzx_sum//2
        G2[t-1] += xyzx_sum * 2**(n-3)
        G2[t]   += error_free * 2**(n-2)
        G2[t+1] += xx_sum * 2**(n-3)
        
        print("# ---------------")
    
    for t in sorted(G2):
        if G2[t] > 0:
            print("%s %s" % (t, G2[t]))


if __name__ == "__main__":
    main()
