from math import factorial

def multinomial (numbers):
    if any(n<0 for n in numbers):
        return 0
    
    denominator = 1
    for n in numbers:
        denominator *= factorial(n)
        
    return (factorial(sum(numbers))//denominator)

def occupations(n,degrees):
    result = [0]*n
    for d in degrees:
        result[d] += 1
    return result

def partitions(n):
    # returns list of partitions of n in lexicographic order
    # base case of recursion: zero is the sum of the empty list
    if n == 0:
        yield []
        return
    # modify partitions of n-1 to form partitions of n
    for p in partitions(n-1):        
        p.append(1)
        yield p
        p.pop()
        if p and (len(p) < 2 or p[-2] > p[-1]):
            p[-1] += 1
            yield p

def degreesequence(partition):
    # returns degree sequence that corresponds to integer partition
    m = 2 + sum(partition)
    result = [d+1 for d in partition]

    while len(result) < m:
        result.append(1)
    return result


        


